#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>

static int * literals;
static int size, count;
static int max, clauses;

static void
die (const char * fmt, ...)
{
  va_list ap;

  fputs ("*** mergecnf: ", stderr);
  va_start (ap, fmt);
  vfprintf (stderr, fmt, ap);
  va_end (ap);
  fputc ('\n', stderr);
  exit (1);
}

static void
parse (const char * name)
{
  FILE * file = fopen (name, "r");
  int ch, sign, lit, new_max;

  if (!file)
    die ("can not read %s", name);

SKIP:

  while (isspace (ch = getc (file)))
    ;

  if (ch == 'c' || ch == 'p')
    {
      while ((ch = getc (file)) != '\n' && ch != EOF)
	;

      goto SKIP;
    }

  new_max = max;

NEXT:

  if (ch == EOF)
    goto DONE;

  if (ch == '-')
    {
      sign = -1;
      ch = getc (file);
    }
  else
    sign = 1;

  if (!isdigit (ch))
    die ("expected digit in %s", name);
  
  lit = ch - '0';
  while (isdigit (ch = getc (file)))
    lit = 10 * lit + (ch - '0');

  if (ch != EOF && !isspace (ch))
    die ("expected EOF or white space after number in %s", name);

  if (lit)
    {
      lit += max;
      if (lit > new_max)
	new_max = lit;

      lit *= sign;
    }
  else
    clauses++;

  if (!literals)
    literals = malloc ((size = 1) * sizeof (literals[0]));
  else if (count >= size)
    literals = realloc (literals, (size *= 2) * sizeof (literals[0]));

  literals[count++] = lit;

  if (ch == EOF)
    goto DONE;
  
  while (isspace (ch = getc (file)))
    ;

  goto NEXT;

DONE:

  fclose (file);
  if (count > 0 && literals[count - 1])
    die ("'0' missing in %s", name);

  max = new_max;
}

static void
print (void)
{
  int *p, *eol = literals + count, lit;

  printf ("p cnf %d %d\n", max, clauses);

  for (p = literals; p < eol; p++)
    {
      lit = *p;
      printf ("%d", lit);
      fputc (lit ? ' ' : '\n', stdout);
    }
}

int
main (int argc, char ** argv)
{
  int i;

  for (i = 1; i < argc; i++)
    parse (argv[i]);

  print ();

  return 0;
}
